<?php

function _phpbb_sso_create_phpbb_session($phpbb_user) {
    if (!($sessions_table = phpbb_get_table('sessions')))
        return;

    module_load_include('inc', 'phpbb_sso', 'phpbb_sso.utility');

    $now = time();
    $values = array(
        ':session_id' => md5(phpbb_sso_unique_id()),
        ':session_user_id' => $phpbb_user->user_id,
        ':session_forum_id' => 0,
        ':session_last_visit' => $now,
        ':session_start' => $now,
        ':session_time' => $now,
        ':session_ip' => '127.0.0.1',
        ':session_browser' => (!empty($_SERVER['HTTP_USER_AGENT'])) ? htmlspecialchars((string) $_SERVER['HTTP_USER_AGENT']) : '',
        ':session_forwarded_for' => (!empty($_SERVER['HTTP_X_FORWARDED_FOR'])) ? htmlspecialchars((string) $_SERVER['HTTP_X_FORWARDED_FOR']) : '',
        ':session_page' => 'index.php',
        ':session_viewonline' => 1,
        ':session_autologin' => 0,
        ':session_admin' => 0, // todo
    );

    db_query("DELETE FROM {$sessions_table} WHERE session_user_id = :user_id OR session_user_id = :anonymous", array(':user_id' => $phpbb_user->user_id, ':anonymous' => 1));
    db_query("INSERT INTO {$sessions_table} (session_id, session_user_id, session_forum_id, session_last_visit, session_start, session_time, session_ip, session_browser, session_forwarded_for, session_page, session_viewonline, session_autologin, session_admin) VALUES (:session_id, :session_user_id, :session_forum_id, :session_last_visit, :session_start, :session_time, :session_ip, :session_browser, :session_forwarded_for, :session_page, :session_viewonline, :session_autologin, :session_admin)", $values);

    $cookie_expire = $now + 31536000; // expires in 365 days

    phpbb_sso_set_cookie('u', $phpbb_user->user_id, $cookie_expire);
    phpbb_sso_set_cookie('k', '', $cookie_expire);
    phpbb_sso_set_cookie('sid', $values[':session_id'], $cookie_expire);

    return true;
}

/**
 * Sets a cookie
 *
 * Sets a cookie of the given name with the specified data for the given length of time. If no time is specified, a session cookie will be set.
 *
 * @param string $name           Name of the cookie, will be automatically prefixed with the phpBB cookie name. track becomes [cookie_name]_track then.
 * @param string $cookiedata     The data to hold within the cookie
 * @param int $cookietime        The expiration time as UNIX timestamp. If 0 is provided, a session cookie is set.
 */
function phpbb_sso_set_cookie($name, $cookiedata, $cookietime) {
    $config = phpbb_sso_get_config(array('cookie_name', 'cookie_path'));

    $name_data = rawurlencode($config['cookie_name'] . '_' . $name) . '=' . rawurlencode($cookiedata);
    $expire = gmdate('D, d-M-Y H:i:s \\G\\M\\T', $cookietime);
    $domain = (!isset($config['cookie_domain']) || $config['cookie_domain'] == 'localhost' || $config['cookie_domain'] == '127.0.0.1') ? '' : '; domain=' . $config['cookie_domain'];

    header('Set-Cookie: ' . $name_data . (($cookietime) ? '; expires=' . $expire : '')
         . '; path=' . $config['cookie_path'] . $domain . ((!isset($config['cookie_secure'])) ? '' : '; secure') . '; HttpOnly', false);
}
